import {Card} from "./Card";

export class Checker {
    private _cards: { rank: string, suite: string }[] = [];
    private _sameRanks: { [key: string]: number } = {};
    private _sameSuites: { [key: string]: number } = {};
    private _checkMethods: ({
        message: string,
        check: (sameRanks: { [key: string]: number }) => boolean
    })[] = [{
        message: "You've got Four of a Kind! 8 points",
        check: this.isFourOfKind
    }, {message: "You've got Full House! 7 points", check: this.isFullHouse}, {
        message: "You've got Street! 5 points",
        check: this.isStreet
    }, {
        message: "You've got Royal Three of a Kind! 4 points",
        check: this.isThreeOfKind
    }, {message: "You've got Two Pare! 3 points", check: this.isTwoPare}, {
        message: "You've got Pare! 2 points",
        check: this.isPare
    }]

    constructor(cards: Card[]) {
        for (const card of cards) {
            this._cards.push({rank: card.rank, suite: card.suite})
        }
        this.sortSameRanks();
        this.sortSameSuites();
    }

    sortSameRanks() {
        if (this._cards) {
            this._sameRanks = this._cards.reduce((acc: { [key: string]: number }, card: {
                rank: string,
                suite: string
            }) => {
                acc[card.rank] = (acc[card.rank]++ || 0) + 1;
                return acc;
            }, {});
        } else {
            console.log("NO cards!")
        }
    }

    sortSameSuites() {
        if (this._cards) {
            this._sameSuites = this._cards.reduce((acc: { [key: string]: number }, card: {
                rank: string,
                suite: string
            }) => {
                acc[card.suite] = (acc[card.suite]++ || 0) + 1;
                return acc;
            }, {});
        } else {
            console.log("NO cards!")
        }
    }

    isPare(sameRanks: { [key: string]: number }) {
        const keysOfSameRanks = Object.keys(sameRanks);
        let check: boolean = false;
        for (const keysOfSameRank of keysOfSameRanks) {
            if (sameRanks[keysOfSameRank] >= 2) {
                check = true;
                break;
            }
        }
        return check;
    }

    isTwoPare(sameRanks: { [key: string]: number }) {
        const keysOfSameRanks = Object.keys(sameRanks);
        let countOfTwos: number = 0;
        if (keysOfSameRanks.length === 3) {
            for (const keysOfSameRank of keysOfSameRanks) {
                if (sameRanks[keysOfSameRank] === 2) {
                    countOfTwos++
                }
            }
        }
        return (countOfTwos === 2);
    }

    isThreeOfKind(sameRanks: { [key: string]: number }) {
        const keysOfSameRanks = Object.keys(sameRanks);
        let check: boolean = false;
        if (keysOfSameRanks.length === 2) {
            for (const keysOfSameRank of keysOfSameRanks) {
                if (this._sameRanks[keysOfSameRank] === 3) {
                    check = true;
                    break;
                }
            }
        }
        return check;
    }

    isFullHouse(sameRanks: { [key: string]: number }) {
        const keysOfSameRanks = Object.keys(sameRanks);
        let check: boolean = false;
        if (keysOfSameRanks.length === 2) {
            for (const keysOfSameRank of keysOfSameRanks) {
                if (this._sameRanks[keysOfSameRank] === 2 || this._sameRanks[keysOfSameRank] === 3) {
                    check = true;
                    break;
                }
            }
        }
        return check;
    }

    isFourOfKind(sameRanks: { [key: string]: number }) {
        const keysOfSameRanks = Object.keys(sameRanks);
        let check: boolean = false;
        for (const keysOfSameRank of keysOfSameRanks) {
            if (sameRanks[keysOfSameRank] === 4) {
                check = true;
                break;
            }
        }
        return check;
    }

    isFlush(sameSuites: { [key: string]: number }) {
        const keysOfSameSuites = Object.keys(sameSuites);
        let check: boolean = false;
        for (const keysOfSameSuite of keysOfSameSuites) {
            if (this._sameSuites[keysOfSameSuite] === 5) {
                check = true;
            }
        }
        return check;
    }

    isStreet(sameRanks: { [key: string]: number }) {
        const streetCombinations = ['23456', '34567', '45678', '56789', '678910', '78910J', '8910JQ', '910JQK']
        const keysOfSameRanks = Object.keys(sameRanks).sort();
        let check: boolean = false;
        if (keysOfSameRanks.length === 5) {
            if (streetCombinations.includes(keysOfSameRanks.join())) {
                check = true;
            }
        }
        return check;
    }

    isRoyalFlush(sameRanks: { [key: string]: number }, sameSuites: { [key: string]: number }) {
        const keysOfSameRanks: string[] = Object.keys(sameRanks).sort();
        const keysOfSameSuites: string[] = Object.keys(sameSuites);
        let check: boolean = false;
        if (keysOfSameRanks.length === 5) {
            if (keysOfSameRanks.join() === "10JQKA" && keysOfSameSuites.length === 1) {
                check = true;
            }
        }
        return check;
    }

    checkCombination() {
        if (this.isRoyalFlush(this._sameRanks, this._sameSuites)) return this.congratMessage("You've got Royal Flash! 10 point!");
        if (this.isFlush(this._sameSuites)) return this.congratMessage("You've got Flash! 6 point!");

        for (const combination of this._checkMethods) {
            if (combination.check(this._sameRanks)) {

                return this.congratMessage(combination.message);
            }
        }
        return this.congratMessage("You've got High Card! 1 point!");
    }

    congratMessage(message: string) {
        return console.log(`
                ==================================================
                *                                                 *
                *   ${message}!!!
                *                                                 *
                ===================================================
                    `)
    }
}